@@ -23,6 +23,18 @@
 #ifndef _COMPIZCONFIG_CCS_BACKEND_CONCEPT_TEST
 #define _COMPIZCONFIG_CCS_BACKEND_CONCEPT_TEST
 
+#ifdef __clang__
+#include <boost/version.hpp>
+#if BOOST_VERSION < 104700
+#error "Boost 1.47 or later is required (http://clang.llvm.org/compatibility.html#deleted-special-func)"
+#endif
+/* Work around a bug in clang which marks functions used by
+ * uninstantiated templates as unused */
+#define COMPIZ_TEMPLATE_EXPECTATION_CHECK COMPIZ_TEMPLATE_EXPECTATION_CHECK
+#else
+#define COMPIZ_TEMPLATE_EXPECTATION_CHECK
+#endif
+
 #include <list>
 
 #include <boost/variant.hpp>
@@ -78,7 +90,7 @@ class CCSBackendConceptTestEnvironmentFactory :
 	CCSBackendConceptTestEnvironmentInterface::Ptr
 	ConstructTestEnv ()
 	{
-	    return boost::shared_static_cast <I> (boost::make_shared <I> ());
+	    return boost::make_shared <I> ();
 	}
 };
 
@@ -175,26 +187,19 @@ namespace compizconfig
     }
 }
 
+using ::testing::AtLeast;
+using ::testing::DoAll;
 using ::testing::Eq;
 using ::testing::IsNull;
-using ::testing::SetArgPointee;
-using ::testing::DoAll;
-using ::testing::Return;
-using ::testing::ReturnNull;
 using ::testing::MakeMatcher;
 using ::testing::Matcher;
 using ::testing::MatcherInterface;
 using ::testing::MatchResultListener;
-using ::testing::AtLeast;
 using ::testing::NiceMock;
-
-namespace
-{
-    bool ccsStringCmp (const CCSString &a, const CCSString &b)
-    {
-	return std::string (a.value) == b.value;
-    }
-}
+using ::testing::NotNull;
+using ::testing::Return;
+using ::testing::ReturnNull;
+using ::testing::SetArgPointee;
 
 namespace cci = compiz::config::impl;
 namespace cc  = compiz::config;
@@ -209,294 +214,6 @@ typedef boost::variant <bool,
 			unsigned int,
 			cci::SettingValueListWrapper::Ptr> VariantTypes;
 
-namespace
-{
-
-typedef boost::function <void ()> WriteFunc;
-
-Bool boolToBool (bool v) { return v ? TRUE : FALSE; }
-
-CCSSettingGMock * getSettingGMockFromSetting (const boost::shared_ptr <CCSSetting> &setting) { return (CCSSettingGMock *) ccsObjectGetPrivate (setting.get ()); }
-
-void SetIntWriteExpectation (const std::string &plugin,
-			     const std::string &key,
-			     const VariantTypes &value,
-			     const boost::shared_ptr <CCSSetting> &setting,
-			     const WriteFunc &write,
-			     const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getInt (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boost::get <int> (value)),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (env->ReadIntegerAtKey (plugin, key), boost::get <int> (value));
-}
-
-void SetBoolWriteExpectation (const std::string &plugin,
-			      const std::string &key,
-			      const VariantTypes &value,
-			      const boost::shared_ptr <CCSSetting> &setting,
-			      const WriteFunc &write,
-			      const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getBool (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boolToBool (boost::get <bool> (value))),
-							 Return (TRUE)));
-    write ();
-
-    bool v (boost::get <bool> (value));
-
-    if (v)
-	EXPECT_THAT (env->ReadBoolAtKey (plugin, key), IsTrue ());
-    else
-	EXPECT_THAT (env->ReadBoolAtKey (plugin, key), IsFalse ());
-}
-
-void SetFloatWriteExpectation (const std::string &plugin,
-			       const std::string &key,
-			       const VariantTypes &value,
-			       const boost::shared_ptr <CCSSetting> &setting,
-			       const WriteFunc &write,
-			       const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getFloat (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boost::get <float> (value)),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (env->ReadFloatAtKey (plugin, key), boost::get <float> (value));
-}
-
-void SetStringWriteExpectation (const std::string &plugin,
-				const std::string &key,
-				const VariantTypes &value,
-				const boost::shared_ptr <CCSSetting> &setting,
-				const WriteFunc &write,
-				const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getString (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     const_cast <char *> (boost::get <const char *> (value))),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (std::string (env->ReadStringAtKey (plugin, key)), std::string (boost::get <const char *> (value)));
-}
-
-void SetColorWriteExpectation (const std::string &plugin,
-			       const std::string &key,
-			       const VariantTypes &value,
-			       const boost::shared_ptr <CCSSetting> &setting,
-			       const WriteFunc &write,
-			       const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getColor (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boost::get <CCSSettingColorValue> (value)),
-							 Return (TRUE)));
-    write ();
-
-    EXPECT_EQ (env->ReadColorAtKey (plugin, key), boost::get <CCSSettingColorValue> (value));
-}
-
-void SetKeyWriteExpectation (const std::string &plugin,
-			     const std::string &key,
-			     const VariantTypes &value,
-			     const boost::shared_ptr <CCSSetting> &setting,
-			     const WriteFunc &write,
-			     const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getKey (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boost::get <CCSSettingKeyValue> (value)),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (env->ReadKeyAtKey (plugin, key), boost::get <CCSSettingKeyValue> (value));
-}
-
-void SetButtonWriteExpectation (const std::string &plugin,
-				const std::string &key,
-				const VariantTypes &value,
-				const boost::shared_ptr <CCSSetting> &setting,
-				const WriteFunc &write,
-				const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getButton (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boost::get <CCSSettingButtonValue> (value)),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (env->ReadButtonAtKey (plugin, key), boost::get <CCSSettingButtonValue> (value));
-}
-
-void SetEdgeWriteExpectation (const std::string &plugin,
-			      const std::string &key,
-			      const VariantTypes &value,
-			      const boost::shared_ptr <CCSSetting> &setting,
-			      const WriteFunc &write,
-			      const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getEdge (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boost::get <unsigned int> (value)),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (env->ReadEdgeAtKey (plugin, key), boost::get <unsigned int> (value));
-}
-
-void SetBellWriteExpectation (const std::string &plugin,
-			      const std::string &key,
-			      const VariantTypes &value,
-			      const boost::shared_ptr <CCSSetting> &setting,
-			      const WriteFunc &write,
-			      const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getBell (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     boolToBool (boost::get <bool> (value))),
-							 Return (TRUE)));
-    write ();
-    bool v (boost::get <bool> (value));
-
-    if (v)
-	EXPECT_THAT (env->ReadBellAtKey (plugin, key), IsTrue ());
-    else
-	EXPECT_THAT (env->ReadBellAtKey (plugin, key), IsFalse ());
-}
-
-void SetMatchWriteExpectation (const std::string &plugin,
-			       const std::string &key,
-			       const VariantTypes &value,
-			       const boost::shared_ptr <CCSSetting> &setting,
-			       const WriteFunc &write,
-			       const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    EXPECT_CALL (*gmock, getMatch (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     const_cast <char *> (boost::get <const char *> (value))),
-							 Return (TRUE)));
-    write ();
-    EXPECT_EQ (std::string (env->ReadMatchAtKey (plugin, key)), std::string (boost::get <const char *> (value)));
-}
-
-void SetListWriteExpectation (const std::string &plugin,
-			      const std::string &key,
-			      const VariantTypes &value,
-			      const boost::shared_ptr <CCSSetting> &setting,
-			      const WriteFunc &write,
-			      const CCSBackendConceptTestEnvironmentInterface::Ptr &env)
-{
-    CCSSettingGMock *gmock (getSettingGMockFromSetting (setting));
-    CCSSettingValueList list = *(boost::get <boost::shared_ptr <cci::SettingValueListWrapper> > (value));
-
-    EXPECT_CALL (*gmock, getInfo ());
-
-    CCSSettingInfo      *info = ccsSettingGetInfo (setting.get ());
-
-    info->forList.listType = (boost::get <boost::shared_ptr <cci::SettingValueListWrapper> > (value))->type ();
-
-    EXPECT_CALL (*gmock, getInfo ()).Times (AtLeast (1));
-    EXPECT_CALL (*gmock, getList (_)).WillRepeatedly (DoAll (
-							 SetArgPointee <0> (
-							     list),
-							 Return (TRUE)));
-    write ();
-
-    EXPECT_THAT (cci::SettingValueListWrapper (env->ReadListAtKey (plugin, key, setting.get ()),
-								   cci::Deep,
-								   info->forList.listType,
-								   boost::shared_ptr <CCSSettingInfo> (),
-								   setting),
-		 ListEqual (&info->forList, list));
-}
-
-void SetIntReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setInt (boost::get <int> (value), _));
-}
-
-void SetBoolReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    bool v (boost::get <bool> (value));
-
-    if (v)
-	EXPECT_CALL (*gmock, setBool (IsTrue (), _));
-    else
-	EXPECT_CALL (*gmock, setBool (IsFalse (), _));
-}
-
-void SetBellReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    bool v (boost::get <bool> (value));
-
-    if (v)
-	EXPECT_CALL (*gmock, setBell (IsTrue (), _));
-    else
-	EXPECT_CALL (*gmock, setBell (IsFalse (), _));
-}
-
-void SetFloatReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setFloat (boost::get <float> (value), _));
-}
-
-void SetStringReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setString (Eq (std::string (boost::get <const char *> (value))), _));
-}
-
-void SetMatchReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setMatch (Eq (std::string (boost::get <const char *> (value))), _));
-}
-
-void SetColorReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setColor (boost::get <CCSSettingColorValue> (value), _));
-}
-
-void SetKeyReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setKey (boost::get <CCSSettingKeyValue> (value), _));
-}
-
-void SetButtonReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setButton (boost::get <CCSSettingButtonValue> (value), _));
-}
-
-void SetEdgeReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    EXPECT_CALL (*gmock, setEdge (boost::get <unsigned int> (value), _));
-}
-
-CCSSettingInfo globalListInfo;
-
-void SetListReadExpectation (CCSSettingGMock *gmock, const VariantTypes &value)
-{
-    globalListInfo.forList.listType = (boost::get <boost::shared_ptr <cci::SettingValueListWrapper> > (value))->type ();
-    globalListInfo.forList.listInfo = NULL;
-
-    ON_CALL (*gmock, getInfo ()).WillByDefault (Return (&globalListInfo));
-    EXPECT_CALL (*gmock, setList (
-			    ListEqual (
-				&globalListInfo.forList,
-				*(boost::get <boost::shared_ptr <cci::SettingValueListWrapper> > (value))), _));
-}
-
-}
-
 class CCSBackendConceptTestParamInterface
 {
     public:
@@ -516,7 +233,7 @@ class CCSBackendConceptTestParamInterface
 				       const compizconfig::test::WriteFunc                  &,
 				       const CCSBackendConceptTestEnvironmentInterface::Ptr &)> SetWriteExpectation;
 
-	virtual void TearDown (CCSBackend *) = 0;
+	virtual void BackendTearDown (CCSBackend *) = 0;
 
 	virtual CCSBackendConceptTestEnvironmentInterface::Ptr testEnv () = 0;
 	virtual VariantTypes & value () = 0;
@@ -559,10 +276,10 @@ class CCSBackendConceptTestParam :
 	{
 	}
 
-	void TearDown (CCSBackend *b)
+	void BackendTearDown (CCSBackend *b)
 	{
 	    if (mTestEnv)
-		mTestEnv->TearDown (b);
+		mTestEnv->BackendTearDown (b);
 
 	    mTestEnv.reset ();
 	}
@@ -634,7 +351,7 @@ class CCSBackendConformanceSpawnObjectsTestFixtureBase
 	    CCSPluginGMock *gmockPlugin = (CCSPluginGMock *) ccsObjectGetPrivate (plugin.get ());
 
 	    ASSERT_FALSE (name.empty ());
-	    ASSERT_TRUE (context.get ());
+	    ASSERT_THAT (context.get (), NotNull ());
 
 	    ON_CALL (*gmockPlugin, getName ()).WillByDefault (Return ((char *) name.c_str ()));
 	    ON_CALL (*gmockPlugin, getContext ()).WillByDefault (Return (context.get ()));
@@ -653,7 +370,7 @@ class CCSBackendConformanceSpawnObjectsTestFixtureBase
 
 	    ASSERT_FALSE (name.empty ());
 	    ASSERT_NE (type, TypeNum);
-	    ASSERT_TRUE (plugin);
+	    ASSERT_THAT (plugin.get (), NotNull ());
 
 	    ON_CALL (*gmockSetting, getName ()).WillByDefault (Return ((char *) name.c_str ()));
 	    ON_CALL (*gmockSetting, getType ()).WillByDefault (Return (type));
@@ -699,13 +416,13 @@ class CCSBackendConformanceTestParameterizedByBackendFixture :
 	virtual void SetUp ()
 	{
 	    SetupContext ();
-	    mBackend = mTestEnv->SetUp (context.get (), gmockContext);
+	    mBackend = mTestEnv->BackendSetUp (context.get (), gmockContext);
 	}
 
 	virtual void TearDown ()
 	{
 	    if (mTestEnv)
-		mTestEnv->TearDown (mBackend);
+		mTestEnv->BackendTearDown (mBackend);
 
 	    mTestEnv.reset ();
 	}
@@ -726,12 +443,12 @@ class CCSBackendConformanceTestParameterized :
 	virtual void SetUp ()
 	{
 	    SetupContext ();
-	    mBackend = GetParam ()->testEnv ()->SetUp (context.get (), gmockContext);
+	    mBackend = GetParam ()->testEnv ()->BackendSetUp (context.get (), gmockContext);
 	}
 
 	virtual void TearDown ()
 	{
-	    CCSBackendConformanceTestParameterized::GetParam ()->TearDown (mBackend);
+	    CCSBackendConformanceTestParameterized::GetParam ()->BackendTearDown (mBackend);
 	}
 };
 
