@@ -258,15 +258,8 @@ class GWDMockSettingsWritableTest :
 {
 };
 
-const GValue referenceGValue = G_VALUE_INIT;
-
 namespace
 {
-    void gwd_settings_notified_unref (GWDSettingsNotified *notified)
-    {
-	g_object_unref (G_OBJECT (notified));
-    }
-
     void gwd_settings_storage_unref (GWDSettingsStorage *storage)
     {
 	g_object_unref (G_OBJECT (storage));
@@ -292,7 +285,11 @@ namespace
 
 	    AutoUnsetGValue (GType type)
 	    {
-		memcpy (&mValue, &referenceGValue, sizeof (GValue));
+		/* This is effectively G_VALUE_INIT, we can't use that here
+		 * because this is not a C++11 project */
+		mValue.g_type = 0;
+		mValue.data[0].v_int = 0;
+		mValue.data[1].v_int = 0;
 		g_value_init (&mValue, type);
 	    }
 
@@ -428,108 +425,153 @@ TEST_F(GWDMockSettingsTest, TestMock)
     EXPECT_CALL (settingsGMock, dispose ());
     EXPECT_CALL (settingsGMock, finalize ());
 
+    /* The order of evaluation of matchers in Google Mock appears to be undefined and
+     * the way GValueMatch is written makes it particularly unsafe when used with
+     * matchers of multiple types on the same function, since there's no guaruntee
+     * that the matchers will be traversed in any order. If a type is passed to
+     * any of the matchers that it doesn't know how to handle then it will
+     * call directly through to GValueCmp which will run into undefined behaviour
+     * in itself.
+     *
+     * In reality, the API for GValueMatch is probably a little bit broken in this
+     * sense, but just satisfying each expectation as soon as its set seems to do
+     * the job here
+     */
+
     /* calling g_object_get_property actually resets
      * the value so expecting 0x0 is correct */
     EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ACTIVE_SHADOW,
 					     GValueMatch <gpointer> (0x0, g_value_get_pointer),
 					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_INACTIVE_SHADOW,
-					     GValueMatch <gpointer> (0x0, g_value_get_pointer),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_USE_TOOLTIPS,
-					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_DRAGGABLE_BORDER_WIDTH,
-					     GValueMatch <gint> (0, g_value_get_int),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ATTACH_MODAL_DIALOGS,
-					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_BLUR_CHANGED,
-					     GValueMatch <gint> (0, g_value_get_int),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_METACITY_THEME,
-					     GValueMatch <const gchar *> (NULL, g_value_get_string),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ACTIVE_OPACITY,
-					     GValueMatch <gdouble> (0.0, g_value_get_double),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_INACTIVE_OPACITY,
-					     GValueMatch <gdouble> (0.0, g_value_get_double),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ACTIVE_SHADE_OPACITY,
-					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_INACTIVE_SHADE_OPACITY,
-					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_BUTTON_LAYOUT,
-					     GValueMatch <const gchar *> (NULL, g_value_get_string),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_ACTION_DOUBLE_CLICK,
-					     GValueMatch <gint> (0, g_value_get_int),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_ACTION_MIDDLE_CLICK,
-					     GValueMatch <gint> (0, g_value_get_int),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_ACTION_RIGHT_CLICK,
-					     GValueMatch <gint> (0, g_value_get_int),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_MOUSE_WHEEL_ACTION,
-					     GValueMatch <gint> (0, g_value_get_int),
-					     _));
-    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_FONT,
-					     GValueMatch <const gchar *> (NULL, g_value_get_string),
-					     _));
 
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "active-shadow",
 			   &pointerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_INACTIVE_SHADOW,
+					     GValueMatch <gpointer> (0x0, g_value_get_pointer),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "inactive-shadow",
 			   &pointerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_USE_TOOLTIPS,
+					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "use-tooltips",
 			   &booleanGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_DRAGGABLE_BORDER_WIDTH,
+					     GValueMatch <gint> (0, g_value_get_int),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "draggable-border-width",
 			   &integerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ATTACH_MODAL_DIALOGS,
+					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "attach-modal-dialogs",
 			   &booleanGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_BLUR_CHANGED,
+					     GValueMatch <gint> (0, g_value_get_int),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "blur",
 			   &integerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_METACITY_THEME,
+					     GValueMatch <const gchar *> (NULL, g_value_get_string),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "metacity-theme",
 			   &stringGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ACTIVE_OPACITY,
+					     GValueMatch <gdouble> (0.0, g_value_get_double),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "metacity-active-opacity",
 			   &doubleGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_INACTIVE_OPACITY,
+					     GValueMatch <gdouble> (0.0, g_value_get_double),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "metacity-inactive-opacity",
 			   &doubleGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_ACTIVE_SHADE_OPACITY,
+					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "metacity-active-shade-opacity",
 			   &booleanGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_INACTIVE_SHADE_OPACITY,
+					     GValueMatch <gboolean> (FALSE, g_value_get_boolean),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "metacity-inactive-shade-opacity",
 			   &booleanGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_BUTTON_LAYOUT,
+					     GValueMatch <const gchar *> (NULL, g_value_get_string),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "metacity-button-layout",
 			   &stringGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_ACTION_DOUBLE_CLICK,
+					     GValueMatch <gint> (0, g_value_get_int),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "titlebar-double-click-action",
 			   &integerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_ACTION_MIDDLE_CLICK,
+					     GValueMatch <gint> (0, g_value_get_int),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "titlebar-middle-click-action",
 			   &integerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_ACTION_RIGHT_CLICK,
+					     GValueMatch <gint> (0, g_value_get_int),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "titlebar-right-click-action",
 			   &integerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_MOUSE_WHEEL_ACTION,
+					     GValueMatch <gint> (0, g_value_get_int),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "mouse-wheel-action",
 			   &integerGValue);
+
+    EXPECT_CALL (settingsGMock, getProperty (GWD_MOCK_SETTINGS_PROPERTY_TITLEBAR_FONT,
+					     GValueMatch <const gchar *> (NULL, g_value_get_string),
+					     _));
+
     g_object_get_property (G_OBJECT (settingsMock.get ()),
 			   "titlebar-font",
 			   &stringGValue);
@@ -1478,7 +1520,7 @@ class GWDMockSettingsStorageFactoryWrapper :
 };
 
 INSTANTIATE_TEST_CASE_P (MockStorageUpdates, GWDSettingsTestStorageUpdates,
-			 ::testing::Values (boost::shared_static_cast <GWDSettingsStorageFactoryWrapperInterface> (boost::make_shared <GWDMockSettingsStorageFactoryWrapper> ())));
+			 ::testing::Values (boost::shared_ptr <GWDSettingsStorageFactoryWrapperInterface> (new GWDMockSettingsStorageFactoryWrapper ())));
 
 #ifdef USE_GSETTINGS
 class GWDSettingsStorageGSettingsTest :
@@ -1630,6 +1672,6 @@ class GWDSettingsStorageGSettingsFactoryWrapper :
 };
 
 INSTANTIATE_TEST_CASE_P (GSettingsStorageUpdates, GWDSettingsTestStorageUpdates,
-			 ::testing::Values (boost::shared_static_cast <GWDSettingsStorageFactoryWrapperInterface> (boost::make_shared <GWDSettingsStorageGSettingsFactoryWrapper> ())));
+			 ::testing::Values (boost::shared_ptr <GWDSettingsStorageFactoryWrapperInterface> (new GWDSettingsStorageGSettingsFactoryWrapper ())));
 
 #endif
