@@ -60,8 +60,16 @@
 #include <QStyle>
 #include <QPainter>
 
+#include <QDBusMessage>
+#include <QDBusConnection>
+#include <QDBusPendingCall>
+
 #include "paintredirector.h"
 
+static const char *KDED_SERVICE = "org.kde.kded";
+static const char *KDED_APPMENU_PATH = "/modules/appmenu";
+static const char *KDED_INTERFACE = "org.kde.kded";
+
 KWD::Window::Window (WId  parentId,
 		     WId  clientId,
 		     WId  frame,
@@ -80,6 +88,7 @@ KWD::Window::Window (WId  parentId,
     mAdvancedMenu (0),
     mOpacityMenu (0),
     mDesktopMenu (0),
+    mAppMenuAvailable (false),
     mProcessKiller (this),
     mKeys (this),
     mResizeOpAction (0),
@@ -217,6 +226,12 @@ KWD::Window::isMaximizable (void) const
     return wInfo.actionSupported (NET::ActionMax);
 }
 
+KDecoration::QuickTileMode
+KWD::Window::quickTileMode (void) const
+{
+    return QuickTileNone;
+}
+
 KDecoration::MaximizeMode
 KWD::Window::maximizeMode (void) const
 {
@@ -367,7 +382,7 @@ KWD::Window::caption (int idx) const
 int
 KWD::Window::tabCount () const
 {
-    return 0;
+    return 1;
 }
 
 long
@@ -554,6 +569,41 @@ KWD::Window::showWindowMenu (const QRect &pos)
     showWindowMenu (pos.bottomLeft ());
 }
 
+void
+KWD::Window::showApplicationMenu (const QPoint &p)
+{
+    QPoint pnt = mDecor->widget ()->mapFromGlobal (p);
+
+    pnt += QPoint (mGeometry.x () - mBorder.left - mPadding.left,
+		   mGeometry.y () - mBorder.top - mPadding.top);
+
+    QList<QVariant> args = QList<QVariant> () << pnt.x () << pnt.y () << qulonglong (mClientId);
+    QDBusMessage method = QDBusMessage::createMethodCall (
+		    KDED_SERVICE, KDED_APPMENU_PATH, KDED_INTERFACE, "showMenu");
+    method.setArguments (args);
+    QDBusConnection::sessionBus ().asyncCall (method);
+}
+
+bool
+KWD::Window::menuAvailable () const
+{
+    return mAppMenuAvailable;
+}
+
+void
+KWD::Window::setAppMenuAvailable ()
+{
+    mAppMenuAvailable = true;
+    emit appMenuAvailable ();
+}
+
+void
+KWD::Window::setAppMenuUnavailable ()
+{
+    mAppMenuAvailable = false;
+    emit appMenuUnavailable ();
+}
+
 KWD::Options::MouseCommand
 KWD::Window::buttonToCommand (Qt::MouseButtons button)
 {
@@ -921,15 +971,17 @@ KWD::Window::buttonToWindowOperation(Qt::MouseButtons button)
 void
 KWD::Window::createDecoration (void)
 {
-    KDecoration *decor;
-
     if (mDecor)
 	return;
 
-    decor = Decorator::pluginManager ()->createDecoration (this);
-    decor->init ();
+    mDecor = Decorator::pluginManager ()->createDecoration (this);
 
-    mDecor = decor;
+    connect(this, SIGNAL(showRequest()), mDecor, SIGNAL(showRequest()));
+    connect(this, SIGNAL(appMenuAvailable()), mDecor, SIGNAL(appMenuAvailable()));
+    connect(this, SIGNAL(appMenuUnavailable()), mDecor, SIGNAL(appMenuUnavailable()));
+    connect(this, SIGNAL(menuHidden()), mDecor, SIGNAL(menuHidden()));
+
+    mDecor->init ();
     mDecor->widget ()->installEventFilter (this);
 
     if (mType != Normal2D)
@@ -941,7 +993,7 @@ KWD::Window::createDecoration (void)
 
     mPadding.top = mPadding.bottom = mPadding.left = mPadding.right = 0;
 
-    if (KDecorationUnstable *deco2 = dynamic_cast<KDecorationUnstable*>(decor))
+    if (KDecorationUnstable *deco2 = dynamic_cast<KDecorationUnstable*>(mDecor))
         deco2->padding (mPadding.left, mPadding.right, mPadding.top, mPadding.bottom);
 
     if (mType == Normal2D && mFrame)
@@ -1529,9 +1581,7 @@ KWD::Window::getWindowProtocols (void)
 
     if (status)
     {
-	int i;
-
-	for (i = 0; i < n; i++)
+	for (int i = 0; i < n; ++i)
 	{
 	    if (p[i] == Atoms::wmTakeFocus)
 		mSupportTakeFocus = true;
@@ -1586,7 +1636,6 @@ KWD::Window::handlePopupAboutToShow (void)
     {
 	NETRootInfo *rootInfo = Decorator::rootInfo ();
 	QString	    name;
-	int	    i;
 	int	    winDesktop = desktop ();
 	QAction     *action;
 	const int   BASE = 10;
@@ -1601,7 +1650,7 @@ KWD::Window::handlePopupAboutToShow (void)
 	action->setChecked (winDesktop == NET::OnAllDesktops);
 	mDesktopMenu->addSeparator ();
 
-	for (i = 1; i <= numberOfDesktops; i++)
+	for (int i = 1; i <= numberOfDesktops; ++i)
 	{
 	    QString basic_name ("%1 %2");
 	    if (i < BASE)
