@@ -70,7 +70,6 @@ WallScreen::drawSwitcherBackground ()
     float           outline = 2.0f;
     int             width, height, radius;
     float           r, g, b, a;
-    unsigned int    i, j;
 
     destroyCairoContext (switcherContext);
     setupCairoContext (switcherContext);
@@ -124,11 +123,11 @@ WallScreen::drawSwitcherBackground ()
     cairo_restore (cr);
 
     cairo_save (cr);
-    for (i = 0; i < (unsigned int) screen->vpSize ().height (); i++)
+    for (unsigned int i = 0; i < (unsigned int) screen->vpSize ().height (); i++)
     {
 	cairo_translate (cr, 0.0, viewportBorder);
 	cairo_save (cr);
-	for (j = 0; j < (unsigned int) screen->vpSize ().width (); j++)
+	for (unsigned int j = 0; j < (unsigned int) screen->vpSize ().width (); j++)
 	{
 	    cairo_translate (cr, viewportBorder, 0.0);
 
@@ -353,12 +352,6 @@ WallScreen::checkDestination (unsigned int destX,
 void
 WallScreen::releaseMoveWindow ()
 {
-    CompWindow *window;
-
-    window = screen->findWindow (moveWindow);
-    if (window)
-	window->syncPosition ();
-
     moveWindow = 0;
 }
 
@@ -502,29 +495,24 @@ WallScreen::handleEvent (XEvent *event)
 
 	    moveViewport (-dx, -dy, None);
 	}
-	if (event->xclient.message_type == Atoms::xdndEnter)
-	{
-	    toggleEdges (true);
-	    edgeDrag = true;
-	}
-	else if (event->xclient.message_type == Atoms::xdndLeave)
-	    edgeDrag = false;
 
 	break;
 
 	case FocusIn:
 	case FocusOut:
-	    if (event->xfocus.mode == NotifyGrab)
-		poller.start ();
-	    else if (event->xfocus.mode == NotifyUngrab)
-		poller.stop ();
-	break;
+	    /* Edges on when grabbed */
+	    if (!optionGetEdgeflipPointer ())
+	    {
+		if (event->xfocus.mode == NotifyGrab)
+		    toggleEdges (true);
+		else if (event->xfocus.mode == NotifyUngrab)
+		    toggleEdges (false);
+	    }
 
 	case ConfigureNotify:
+	break;
 
-	     if (event->xconfigure.window == screen->root ())
-		updateScreenEdgeRegions ();
-
+	default:
 	break;
     }
 
@@ -532,67 +520,32 @@ WallScreen::handleEvent (XEvent *event)
 }
 
 /*
- * Borrowed this from PrivateScreen::updateScreenEdges
- *
+ * When a dnd type window is mapped toggle edge flip dnd windows back on
+ * _NET_WM_WINDOW_TYPE_DND windows. This will not detect all dnd windows
+ * but it will detect the EWMH-compliant ones without resorting to
+ * hacks involving selection stealing.
  */
-
-#define SCREEN_EDGE_NUM		8
-
 void
-WallScreen::updateScreenEdgeRegions ()
+WallWindow::windowNotify (CompWindowNotify n)
 {
-    edgeRegion = CompRegion (0, 0, screen->width (), screen->height ());
-    noEdgeRegion = CompRegion (0, 0, screen->width (), screen->height ());
+    WallScreen *ws = WallScreen::get (screen);
+    bool toggleOnDnd = ws->optionGetEdgeflipDnd ();
 
-    struct screenEdgeGeometry {
-	int xw, x0;
-	int yh, y0;
-	int ww, w0;
-	int hh, h0;
-    } geometry[SCREEN_EDGE_NUM] = {
-	{ 0,  0,   0,  2,   0,  2,   1, -4 }, /* left */
-	{ 1, -2,   0,  2,   0,  2,   1, -4 }, /* right */
-	{ 0,  2,   0,  0,   1, -4,   0,  2 }, /* top */
-	{ 0,  2,   1, -2,   1, -4,   0,  2 }, /* bottom */
-	{ 0,  0,   0,  0,   0,  2,   0,  2 }, /* top-left */
-	{ 1, -2,   0,  0,   0,  2,   0,  2 }, /* top-right */
-	{ 0,  0,   1, -2,   0,  2,   0,  2 }, /* bottom-left */
-	{ 1, -2,   1, -2,   0,  2,   0,  2 }  /* bottom-right */
-    };
-
-    for (unsigned int i = 0; i < SCREEN_EDGE_NUM; i++)
+    switch (n)
     {
-	CompRegion edge (geometry[i].xw * screen->width () +
-			 geometry[i].x0,
-			 geometry[i].yh * screen->height () +
-			 geometry[i].y0,
-			 geometry[i].ww * screen->width () +
-			 geometry[i].w0,
-			 geometry[i].hh * screen->height () +
-			 geometry[i].h0);
-
-	noEdgeRegion -= edgeRegion;
+	case CompWindowNotifyMap:
+	    if (window->type () & CompWindowTypeDndMask && toggleOnDnd)
+		ws->toggleEdges (true);
+	    break;
+	case CompWindowNotifyUnmap:
+	    if (window->type () & CompWindowTypeDndMask && toggleOnDnd)
+		ws->toggleEdges (false);
+	    break;
+	default:
+	    break;
     }
 
-    edgeRegion -= noEdgeRegion;
-}
-
-#undef SCREEN_EDGE_NUM
-
-void
-WallScreen::positionUpdate (const CompPoint &pos)
-{
-    if (edgeDrag)
-	return;
-
-    if (edgeRegion.contains (pos))
-	toggleEdges (false);
-    else if (noEdgeRegion.contains (pos))
-    {
-	if (!screen->grabbed ())
-	    poller.stop ();
-	toggleEdges (true);
-    }
+    window->windowNotify (n);
 }
 
 void
@@ -609,6 +562,10 @@ WallWindow::activate ()
 	dx       = viewport.x ();
 	dy       = viewport.y ();
 
+	/* Handle negative value */
+	dx = (unsigned int) dx % screen->vpSize ().width ();
+	dy = (unsigned int) dy % screen->vpSize ().height ();
+
 	dx -= screen->vp ().x ();
 	dy -= screen->vp ().y ();
 
@@ -625,23 +582,26 @@ WallWindow::activate ()
 		return;
 	    }
 
-	    ws->focusDefault = false;
+	    if (ws->optionGetAutoSwitchVpAndWindow ())
+	    {
+		ws->focusDefault = false;
 
-	    CompRegion screenRegion;
+		CompRegion screenRegion;
 
-	    foreach (const CompOutput &o, screen->outputDevs ())
-		screenRegion += o.workArea ();
+		foreach (const CompOutput &o, screen->outputDevs ())
+		    screenRegion += o.workArea ();
 
-	    CompPoint d = compiz::wall::movementWindowOnScreen (window->serverBorderRect (),
-								screenRegion);
+		CompPoint d = compiz::wall::movementWindowOnScreen (window->serverBorderRect (),
+								    screenRegion);
 
-	    mask |= d.x () !=0 ? CWX : 0;
-	    mask |= d.y () !=0 ? CWY : 0;
+		mask |= d.x () !=0 ? CWX : 0;
+		mask |= d.y () !=0 ? CWY : 0;
 
-	    xwc.x = window->serverGeometry ().x () + dx;
-	    xwc.y = window->serverGeometry ().y () + dy;
+		xwc.x = window->serverGeometry ().x () + d.x ();
+		xwc.y = window->serverGeometry ().y () + d.y ();
 
-	    window->configureXWindow (mask, &xwc);
+		window->configureXWindow (mask, &xwc);
+	    }
 	}
     }
 
@@ -651,19 +611,19 @@ WallWindow::activate ()
 void
 WallWindow::grabNotify (int          x,
 			int          y,
-			unsigned int width,
-			unsigned int height)
+			unsigned int state,
+			unsigned int mask)
 {
-    WallScreen::get (screen)->toggleEdges (true);
-    WallScreen::get (screen)->edgeDrag = true;
+    if (mask & (CompWindowGrabMoveMask | CompWindowGrabButtonMask))
+	WallScreen::get (screen)->windowIsDragMoved = true;
 
-    window->grabNotify (x, y, width, height);
+    window->grabNotify (x, y, state, mask);
 }
 
 void
 WallWindow::ungrabNotify ()
 {
-    WallScreen::get (screen)->edgeDrag = false;
+    WallScreen::get (screen)->windowIsDragMoved = false;
 
     window->ungrabNotify ();
 }
@@ -807,52 +767,47 @@ bool
 WallScreen::initiateFlip (Direction         direction,
 			  CompAction::State state)
 {
+    CompOption::Vector o (0);
+
     int dx, dy;
     int amountX, amountY;
 
-    if (screen->otherGrabExist ("wall", "move", "group-drag", 0))
+    const bool allowFlipDnd = (state & CompAction::StateInitEdgeDnd) &&
+			      optionGetEdgeflipDnd ();
+    const bool allowFlipMove = (windowIsDragMoved &&
+				optionGetEdgeflipMove ());
+    const bool allowFlipPointer = optionGetEdgeflipPointer ();
+
+    if (!allowFlipDnd &&
+	!allowFlipMove &&
+	!allowFlipPointer)
 	return false;
 
-    if (state & CompAction::StateInitEdgeDnd)
+    switch (direction)
     {
-	if (!optionGetEdgeflipDnd ())
-	    return false;
-
-	if (screen->otherGrabExist ("wall", 0))
-	    return false;
-    }
-    else if (screen->grabExist ("move"))
-    {
-	if (!optionGetEdgeflipMove ())
-	    return false;
-    }
-    else if (screen->grabExist ("group-drag"))
-    {
-	if (!optionGetEdgeflipDnd ())
-	    return false;
-    }
-    else if (!optionGetEdgeflipPointer ())
-    {
-	toggleEdges (false);
-	poller.start ();
-	return false;
-    }
-
-    switch (direction) {
     case Left:
-	dx = -1; dy = 0;
+	dx = -1;
+	dy = 0;
 	break;
+
     case Right:
-	dx = 1; dy = 0;
+	dx = 1;
+	dy = 0;
 	break;
+
     case Up:
-	dx = 0; dy = -1;
+	dx = 0;
+	dy = -1;
 	break;
+
     case Down:
-	dx = 0; dy = 1;
+	dx = 0;
+	dy = 1;
 	break;
+
     default:
-	dx = 0; dy = 0;
+	dx = 0;
+	dy = 0;
 	break;
     }
 
@@ -864,7 +819,7 @@ WallScreen::initiateFlip (Direction         direction,
 
 	if (dx < 0)
 	{
-	    offsetX = screen->width () - 10;
+	    offsetX = screen->width () - 1;
 	    warpX = pointerX + screen->width ();
 	}
 	else if (dx > 0)
@@ -880,7 +835,7 @@ WallScreen::initiateFlip (Direction         direction,
 
 	if (dy < 0)
 	{
-	    offsetY = screen->height () - 10;
+	    offsetY = screen->height () - 1;
 	    warpY = pointerY + screen->height ();
 	}
 	else if (dy > 0)
@@ -949,7 +904,6 @@ WallScreen::drawCairoTextureOnScreen (const GLMatrix &transform)
     float             width, height;
     float             topLeftX, topLeftY;
     float             border;
-    unsigned int      i, j;
     GLTexture::Matrix matrix;
     BOX               box;
     GLMatrix          wTransform (transform);
@@ -1020,9 +974,9 @@ WallScreen::drawCairoTextureOnScreen (const GLMatrix &transform)
     height = (float) thumbContext.height;
 
     thumbContext.texture[0]->enable (GLTexture::Fast);
-    for (i = 0; i < (unsigned int) screen->vpSize ().width (); i++)
+    for (unsigned int i = 0; i < (unsigned int) screen->vpSize ().width (); i++)
     {
-	for (j = 0; j < (unsigned int) screen->vpSize ().height (); j++)
+	for (unsigned int j = 0; j < (unsigned int) screen->vpSize ().height (); j++)
 	{
 	    if (i == gotoX && j == gotoY && moving)
 		continue;
@@ -1197,7 +1151,6 @@ WallScreen::glPaintOutput (const GLScreenPaintAttrib &attrib,
 
 	if (optionGetMiniscreen ())
         {
-	    unsigned int i, j;
 	    float        mw, mh;
 
 	    mw = viewportWidth;
@@ -1209,9 +1162,9 @@ WallScreen::glPaintOutput (const GLScreenPaintAttrib &attrib,
 	    mSAttribs.opacity = OPAQUE * (1.0 + mSzCamera);
 	    mSAttribs.saturation = COLOR;
 
-	    for (j = 0; j < (unsigned int) screen->vpSize ().height (); j++)
+	    for (unsigned int j = 0; j < (unsigned int) screen->vpSize ().height (); j++)
 	    {
-		for (i = 0; i < (unsigned int) screen->vpSize ().width (); i++)
+		for (unsigned int i = 0; i < (unsigned int) screen->vpSize ().width (); i++)
 		{
 		    float        mx, my;
 		    unsigned int msMask;
@@ -1618,6 +1571,9 @@ WallScreen::optionChanged (CompOption           *opt,
 	    ww->isSliding = !optionGetNoSlideMatch ().evaluate (w);
 	}
 	break;
+    case WallOptions::EdgeflipPointer:
+	toggleEdges (optionGetEdgeflipPointer ());
+	break;
 
     default:
 	break;
@@ -1678,7 +1634,7 @@ WallScreen::WallScreen (CompScreen *screen) :
     moveWindow (None),
     focusDefault (true),
     transform (NoTransformation),
-    edgeDrag (false)
+    windowIsDragMoved (false)
 {
     ScreenInterface::setHandler (screen);
     CompositeScreenInterface::setHandler (cScreen);
@@ -1745,11 +1701,6 @@ WallScreen::WallScreen (CompScreen *screen) :
     setNotify (ArrowShadowColor);
     setNotify (NoSlideMatch);
     setNotify (EdgeflipPointer);
-
-    updateScreenEdgeRegions ();
-
-    poller.setCallback (boost::bind (&WallScreen::positionUpdate, this,
-				     _1));
 }
 
 WallScreen::~WallScreen ()
@@ -1776,15 +1727,10 @@ WallWindow::WallWindow (CompWindow *window) :
 bool
 WallPluginVTable::init ()
 {
-    if (!CompPlugin::checkPluginABI ("core", CORE_ABIVERSION))
-        return false;
-    if (!CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI))
-	return false;
-    if (!CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI))
-	return false;
-    if (!CompPlugin::checkPluginABI ("mousepoll", COMPIZ_MOUSEPOLL_ABI))
-	return false;
+    if (CompPlugin::checkPluginABI ("core", CORE_ABIVERSION)		&&
+	CompPlugin::checkPluginABI ("composite", COMPIZ_COMPOSITE_ABI)	&&
+	CompPlugin::checkPluginABI ("opengl", COMPIZ_OPENGL_ABI))
+	return true;
 
-    return true;
+    return false;
 }
-
