@@ -43,56 +43,12 @@
 #include "privateiosource.h"
 #include "privateeventsource.h"
 #include "privatesignalsource.h"
+#include "outputdevices.h"
 
 #include "core_options.h"
 
 #include <set>
 
-namespace compiz { namespace private_screen
-{
-
-class OutputDevices
-{
-public:
-    OutputDevices();
-
-    void setCurrentOutput(unsigned int outputNum);
-
-    CompOutput& getCurrentOutputDev() { return outputDevs[currentOutputDev]; }
-
-    bool hasOverlappingOutputs() const { return overlappingOutputs; }
-
-    void computeWorkAreas(CompRect& workArea, bool& workAreaChanged,
-	    CompRegion& allWorkArea, const CompWindowList& windows);
-
-    const CompOutput& getOutputDev(unsigned int outputNum) const
-    { return outputDevs[outputNum]; }
-
-    // TODO breaks encapsulation horribly ought to be const at least
-    // Even better, use begin() and end() return const_iterators
-    // BUT this is exported directly through API - which makes changing
-    // it a PITA.
-    CompOutput::vector& getOutputDevs() { return outputDevs; }
-
-    int outputDeviceForGeometry(const CompWindow::Geometry& gm, int strategy,
-	    CompScreen* screen) const;
-    void updateOutputDevices(CoreOptions& coreOptions, CompScreen* screen);
-
-private:
-    void setGeometryOnDevice(unsigned int nOutput, int x, int y,
-	    const int width, const int height);
-    void adoptDevices(unsigned int nOutput);
-
-    static CompRect computeWorkareaForBox(const CompRect& box,
-	    const CompWindowList& windows);
-
-    CompOutput::vector outputDevs;
-    bool               overlappingOutputs;
-    int	           currentOutputDev;
-};
-
-}} //::compiz::private_screen
-
 CompPlugin::VTable * getCoreVTable ();
 
 class CoreWindow;
@@ -221,6 +177,11 @@ class WindowManager : boost::noncopyable
 	reverse_iterator rbegin() const { return windows.rbegin(); }
 	reverse_iterator rend() const { return windows.rend(); }
 
+	iterator serverBegin() const { return serverWindows.begin(); }
+	iterator serverEnd() const { return serverWindows.end(); }
+	reverse_iterator rserverBegin() const { return serverWindows.rbegin(); }
+	reverse_iterator rserverEnd() const { return serverWindows.rend(); }
+
 	void clearFullscreenHints() const;
 	void showOrHideForDesktop(unsigned int desktop) const;
 	void setWindowActiveness(::compiz::private_screen::History& history) const;
@@ -353,14 +314,11 @@ class EventManager :
 
 	Glib::RefPtr <Glib::MainLoop>  mainloop;
 
-	/* We cannot use RefPtrs. See
-	 * https://bugzilla.gnome.org/show_bug.cgi?id=561885
-	 */
-	CompEventSource * source;
-	CompTimeoutSource * timeout;
-	CompSignalSource * sighupSource;
-	CompSignalSource * sigtermSource;
-	CompSignalSource * sigintSource;
+	Glib::RefPtr <CompEventSource> source;
+	Glib::RefPtr <CompTimeoutSource> timeout;
+	CompSignalSource *sighupSource;
+	CompSignalSource *sigtermSource;
+	CompSignalSource *sigintSource;
 	Glib::RefPtr <Glib::MainContext> ctx;
 
 	CompFileWatchList   fileWatch;
@@ -368,7 +326,7 @@ class EventManager :
 
 	// TODO - almost certainly the wrong data structure
 	// Why not a std::map<CompWatchFdHandle, CompWatchFd>?
-	std::list< CompWatchFd * > watchFds;
+	std::list<Glib::RefPtr<CompWatchFd> > watchFds;
 	CompWatchFdHandle        lastWatchFdHandle;
 
         bool	grabbed;   /* true once we receive a GrabNotify
@@ -427,8 +385,12 @@ public:
     void updatePassiveKeyGrabs ();
     void updatePassiveButtonGrabs(Window serverFrame);
 
+    void setCurrentState(CompAction::State state);
+
 private:
     CompScreen  * const screen;
+    CompAction::State currentState;
+
     std::list<ButtonGrab> buttonGrabs;
     std::list<KeyGrab>    keyGrabs;
 };
@@ -461,7 +423,7 @@ class ViewportRetrievalInterface
 	virtual ~ViewportRetrievalInterface () {}
 
 	virtual const CompPoint & getCurrentViewport () const = 0;
-	virtual const CompSize & viewportDimentions () const = 0;
+	virtual const CompSize & viewportDimensions () const = 0;
 };
 
 // Apart from a use by StartupSequence::addSequence this data
@@ -478,7 +440,7 @@ struct ViewPort :
     private:
 
 	const CompPoint & getCurrentViewport () const { return vp; }
-	const CompSize & viewportDimentions () const { return vpSize; }
+	const CompSize & viewportDimensions () const { return vpSize; }
 };
 
 namespace viewports
@@ -767,7 +729,7 @@ public:
     compiz::private_screen::StartupSequenceImpl startupSequence;
     compiz::private_screen::EventManager eventManager;
     compiz::private_screen::OrphanData orphanData;
-    compiz::private_screen::OutputDevices outputDevices;
+    compiz::core::OutputDevices outputDevices;
 
     Colormap colormap;
     int screenNum;
@@ -779,6 +741,7 @@ public:
 
     Window wmSnSelectionWindow;
 
+    int    clientPointerDeviceId;
     Cursor normalCursor;
     Cursor busyCursor;
     Cursor invisibleCursor;
@@ -1106,6 +1069,10 @@ class CompScreenImpl : public CompScreen,
 				   CompAction::State  state,
 				   CompOption::Vector &options);
 
+	static bool unmaximizeOrMinimizeWin (CompAction         *action,
+					     CompAction::State  state,
+					     CompOption::Vector &options);
+
 	static bool minimizeWin (CompAction         *action,
 				 CompAction::State  state,
 				 CompOption::Vector &options);
